package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.NeoPushDTO;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.DiseaseTypeEnum;
import com.diagbot.enums.MedicalAdviceEnum;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.process.PushProcess;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 推送facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class PushFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    PushProcess pushProcess;

    /**
     * 推送业务
     * @param
     * @return
     */
    public PushDTO pushFac(PushVO pushVo) {
        PushDTO pushDTO = new PushDTO();
        // 模型处理数据
        WordCrfDTO wordCrfDTO = commonFacade.crf_process(pushVo);

        // 标准词转换
        StandConvert standConvert = commonFacade.dataTypeGet(wordCrfDTO);
        Map<String, Map<String, String>> standConvertMap = neoFacade.standConvertCrf(standConvert);
        commonFacade.dataTypeSet(wordCrfDTO, standConvertMap);
        List<String> ruleTypeList = Arrays.asList(pushVo.getFeatureType().split(","));
        Map<String, List<String>> typeWords = typeWords(standConvertMap);
        //生成push入参，供图谱调用
        NeoPushVO pushVO = commonFacade.generatePushInput(wordCrfDTO);
        //图谱推送数据
        List<NeoPushDTO> push = neoFacade.getPush(pushVO);
       /* NeoPushDTO neoPushDTO1 = new NeoPushDTO();
        PushBaseDTO disease = new PushBaseDTO();
        disease.setName("急性胰腺炎");
        neoPushDTO1.setDisease(disease);
        List<PushBaseDTO> vitals = new ArrayList<>();
        PushBaseDTO vital1 = new PushBaseDTO();
        vital1.setName("心率快");
        vitals.add(vital1);
        PushBaseDTO vital2 = new PushBaseDTO();
        vital2.setName("淋巴肿大");
        vitals.add(vital2);
        neoPushDTO1.setVitals(vitals);
        push.add(neoPushDTO1);*/


        int length = pushVo.getLength();
        if(ListUtil.isNotEmpty(push)){
            NeoPushDTO neoPushDTO = push.get(0);
            if(neoPushDTO != null){
                // 症状
                if (ruleTypeList.contains("1") && ListUtil.isNotEmpty(neoPushDTO.getSymptoms())) {
                    List<PushBaseDTO> filterSymptoms = neoPushDTO.getSymptoms().stream().filter(x -> !typeWords.get(StandConvertEnum.symptom.getName()).contains(x.getName())).collect(Collectors.toList());
                    pushDTO.setSymptom(filterSymptoms.subList(0, filterSymptoms.size() >= length ? length : filterSymptoms.size()));
                }
                // 查体
                if (ruleTypeList.contains("4") && ListUtil.isNotEmpty(neoPushDTO.getVitals())) {
                    List<PushBaseDTO> filtervitals = neoPushDTO.getVitals().stream().filter(x -> !typeWords.get(StandConvertEnum.vital.getName()).contains(x.getName())).collect(Collectors.toList());
                    pushDTO.setVital(filtervitals.subList(0, filtervitals.size() >= length ? length : filtervitals.size()));
                }
                // 化验
                if (ruleTypeList.contains("5") && ListUtil.isNotEmpty(neoPushDTO.getLis())) {
                    List<PushBaseDTO> filterlis = neoPushDTO.getLis().stream().filter(x -> !typeWords.get(StandConvertEnum.lis.getName()).contains(x.getName())).collect(Collectors.toList());
                    pushDTO.setLis(filterlis.subList(0, filterlis.size() >= length ? length : filterlis.size()));
                }
                // 辅检
                if (ruleTypeList.contains("6") && ListUtil.isNotEmpty(neoPushDTO.getPacs())) {
                    List<PushBaseDTO> filterpacs = neoPushDTO.getPacs().stream().filter(x -> !typeWords.get(StandConvertEnum.pacs.getName()).contains(x.getName())).collect(Collectors.toList());
                    pushDTO.setPacs(filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size()));
                }
                // 药品
                if (ruleTypeList.contains("8") && neoPushDTO.getTreat() != null && ListUtil.isNotEmpty(neoPushDTO.getTreat().get(MedicalAdviceEnum.drug.getName()))) {
                    List<PushBaseDTO> pushMedicineBaseDTOS = neoPushDTO.getTreat().get(MedicalAdviceEnum.drug.getName());
                    pushDTO.setMedicines(pushMedicineBaseDTOS.subList(0, pushMedicineBaseDTOS.size() >= length ? length : pushMedicineBaseDTOS.size()));
                }
                // 手术
                if (ruleTypeList.contains("9") && neoPushDTO.getTreat() != null && ListUtil.isNotEmpty(neoPushDTO.getTreat().get(MedicalAdviceEnum.operation.getName()))) {
                    List<PushBaseDTO> pushOperationBaseDTOS = neoPushDTO.getTreat().get(MedicalAdviceEnum.operation.getName());
                    pushDTO.setMedicines(pushOperationBaseDTOS.subList(0, pushOperationBaseDTOS.size() >= length ? length : pushOperationBaseDTOS.size()));
                }
            }
            Map<String, List<PushBaseDTO>> dis = new HashMap<>();
            //推送诊断
            if (ruleTypeList.contains("7")) {
                List<PushBaseDTO> fiterDiseases = push.stream().map(x -> x.getDisease()).filter(y -> !typeWords.get(StandConvertEnum.disease.getName()).contains(y.getName())).collect(Collectors.toList());
                List<PushBaseDTO> pushBaseDTOS = fiterDiseases.subList(0, fiterDiseases.size() >= length ? length : fiterDiseases.size());
                dis.put(DiseaseTypeEnum.possibleDis.getName(),pushBaseDTOS);

                List<List<String>> differenrDisList = push.stream().filter(x -> ListUtil.isNotEmpty(x.getDifferentialDis())).map(y -> y.getDifferentialDis().stream().map(
                        z -> z.getName()).collect(Collectors.toList())).collect(Collectors.toList());
                List<String> differentDises = new ArrayList<>();
                differenrDisList.forEach(
                        x ->{
                            differentDises.removeAll(x);
                            differentDises.addAll(x);
                        }
                );
                List<PushBaseDTO> filterDifferentDis = differentDises.subList(0, differentDises.size() >= length ? length : differentDises.size()).stream().map(x -> {
                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                    pushBaseDTO.setName(x);
                    return pushBaseDTO;
                }).collect(Collectors.toList());
                dis.put(DiseaseTypeEnum.identify.getName(),filterDifferentDis);
            }
            pushDTO.setDis(dis);

        }

        /*if (wordCrfDTO.getDiagLabel() != null && ListUtil.isNotEmpty(wordCrfDTO.getDiagLabel().getDiags())) { // 有诊断
            List<String> diags = wordCrfDTO.getDiagLabel().getDiags().stream().map(r -> r.getStandName()).collect(Collectors.toList());
            PushNeoVO pushNeoVO = new PushNeoVO();
            pushNeoVO.setSex(pushVo.getSex());
            pushNeoVO.setAge(pushVo.getAge());
            pushNeoVO.setDiags(diags);
            Map<String, PushDTO> map = neoFacade.getInverse(pushNeoVO);
            // 获取第一个诊断
            String firstDisease = diags.get(0);
            PushDTO neoDto = map.get(firstDisease);

            if (neoDto != null) {
                int length = pushVo.getLength();
                // 症状
                if (ruleTypeList.contains("1") && ListUtil.isNotEmpty(neoDto.getSymptom())) {
                    pushDTO.setSymptom(neoDto.getSymptom().subList(0, neoDto.getSymptom().size() >= length ? length : neoDto.getSymptom().size()));
                }

                // 查体
                if (ruleTypeList.contains("4") && ListUtil.isNotEmpty(neoDto.getVital())) {
                    pushDTO.setVital(neoDto.getVital().subList(0, neoDto.getVital().size() >= length ? length : neoDto.getVital().size()));
                }

                // 化验
                if (ruleTypeList.contains("5") && ListUtil.isNotEmpty(neoDto.getLis())) {
                    pushDTO.setLis(neoDto.getLis().subList(0, neoDto.getLis().size() >= length ? length : neoDto.getLis().size()));
                }

                // 辅检
                if (ruleTypeList.contains("6") && ListUtil.isNotEmpty(neoDto.getPacs())) {
                    pushDTO.setPacs(neoDto.getPacs().subList(0, neoDto.getPacs().size() >= length ? length : neoDto.getPacs().size()));
                }

                // 诊断
                if (ruleTypeList.contains("7") && neoDto.getDis() != null) {
                    Map<String, List<PushBaseDTO>> disMap = neoDto.getDis();
                    Map<String, List<PushBaseDTO>> resMap = new HashMap<>();
                    for (String key : disMap.keySet()) {
                        resMap.put(key, disMap.get(key).subList(0, disMap.get(key).size() >= length ? length : disMap.get(key).size()));
                    }
                    pushDTO.setDis(resMap);
                }
            }

        }*/
        return pushDTO;
    }

    public BillNeoVO fillBillNeo(IndicationPushVO indicationPushVO) {
        BillNeoVO billNeoVO = new BillNeoVO();

        List<Item> items = indicationPushVO.getDrug();
        for (Item item : items) {
            billNeoVO.getDrugList().put(item.getName(), item.getUniqueName());
        }

        return billNeoVO;
    }

    public Map<String,List<String>> typeWords( Map<String, Map<String, String>> standConvertMap){
        Map<String,List<String>> typeWordsMap = new HashMap<>();
        for (Map.Entry<String, Map<String, String>> s:standConvertMap.entrySet()) {
            List<String> words = new ArrayList<>();
            String type = s.getKey();
            Map<String, String> value = s.getValue();
            value.forEach((name,standName)->{
                words.add(name);
                words.add(standName);
            });
            typeWordsMap.put(type,new ArrayList<>(new HashSet<>(words)));
        }

        return typeWordsMap;
    }
}
